@@ -23,7 +23,7 @@ import re
 import subprocess
 import tempfile
 import urllib.error
-from urllib.parse import quote, urlparse, urlunparse, urljoin
+from urllib.parse import quote, urlparse, urlunparse, urljoin, parse_qs
 from warnings import warn
 
 from debian.deb822 import Deb822
@@ -34,6 +34,7 @@ from lintian_brush import (
     SUPPORTED_CERTAINTIES,
     certainty_sufficient,
     certainty_to_confidence,
+    version_string,
     )
 from lintian_brush.vcs import (
     browse_url_from_repo_url,
@@ -181,6 +182,14 @@ def guess_repo_from_url(url, net_access=False):
             parts = parts[:parts.index('tags')]
         return urlunparse(
             parsed_url._replace(path='/'.join(parts), query=''))
+    if parsed_url.hostname == 'git.php.net':
+        if parsed_url.path.startswith('/repository/'):
+            return url
+        if not parsed_url.path.strip('/'):
+            qs = parse_qs(parsed_url.query)
+            if 'p' in qs:
+                return urlunparse(parsed_url._replace(
+                    path='/repository/' + qs['p'][0], query=''))
     if parsed_url.netloc in KNOWN_HOSTING_SITES:
         return url
     # Maybe it's already pointing at a VCS repo?
@@ -231,8 +240,8 @@ def get_python_pkg_info(path, trust_package=False):
         return pkg_info
     filename = os.path.join(path, 'setup.py')
     args = [os.path.abspath(filename), 'dist_info']
-    with open(filename, 'r') as f:
-        has_shebang = f.readline().startswith('#!')
+    with open(filename, 'rb') as f:
+        has_shebang = f.readline().startswith(b'#!')
     is_executable = (os.stat(filename).st_mode & 0o100 != 0)
     if not has_shebang or not is_executable:
         # TODO(jelmer): Why python3 and not e.g. python?
@@ -436,6 +445,9 @@ def guess_from_debian_copyright(path, trust_package):
             if repo_url:
                 yield UpstreamDatum(
                     'Repository', sanitize_vcs_url(repo_url), 'likely')
+            if (from_url.startswith('https://pecl.php.net/package/') or
+                    from_url.startswith('http://pecl.php.net/package/')):
+                yield UpstreamDatum('X-Pecl-URL', from_url, 'certain')
         if "X-Upstream-Bugs" in header:
             yield UpstreamDatum(
                 "Bug-Database", header["X-Upstream-Bugs"], 'certain')
@@ -672,7 +684,7 @@ def guess_from_configure(path, trust_package=False):
 
 
 def guess_from_r_description(path, trust_package=False):
-    with open(path, 'r') as f:
+    with open(path, 'rb') as f:
         description = Deb822(f)
         if 'Package' in description:
             yield UpstreamDatum('Name', description['Package'], 'certain')
@@ -881,6 +893,14 @@ def extend_from_sf(upstream_metadata, sf_project):
         guess_from_sf(sf_project))
 
 
+def extend_from_pecl(upstream_metadata, pecl_url, certainty):
+    pecl_fields = ['Homepage', 'Repository', 'Bug-Database']
+
+    return extend_from_external_guesser(
+        upstream_metadata, certainty, pecl_fields,
+        guess_from_pecl_url(pecl_url))
+
+
 def extend_from_lp(upstream_metadata, minimum_certainty, package,
                    distribution=None, suite=None):
     # The set of fields that Launchpad can possibly provide:
@@ -1129,6 +1149,9 @@ def extend_upstream_metadata(upstream_metadata, path, minimum_certainty=None,
         else:
             extend_from_lp(upstream_metadata, minimum_certainty, package)
         extend_from_aur(upstream_metadata, minimum_certainty, package)
+    pecl_url = upstream_metadata.get('X-Pecl-URL')
+    if net_access and pecl_url:
+        extend_from_pecl(upstream_metadata, pecl_url.value, pecl_url.certainty)
     changed = True
     while changed:
         changed = False
@@ -1282,6 +1305,44 @@ def parse_pkgbuild_variables(f):
     return variables
 
 
+def guess_from_pecl(package):
+    if not package.startswith('php-'):
+        return iter([])
+    php_package = package[4:]
+    url = 'https://pecl.php.net/packages/%s' % php_package.replace('-', '_')
+    data = dict(guess_from_pecl_url(url))
+    try:
+        data['Repository'] = guess_repo_from_url(data['Repository-Browse'])
+    except KeyError:
+        pass
+    return data.items()
+
+
+def guess_from_pecl_url(url):
+    headers = {'User-Agent': USER_AGENT}
+    try:
+        f = urlopen(
+            Request(url, headers=headers),
+            timeout=DEFAULT_URLLIB_TIMEOUT)
+    except urllib.error.HTTPError as e:
+        if e.status != 404:
+            raise
+        return
+    from bs4 import BeautifulSoup
+    bs = BeautifulSoup(f.read(), features='lxml')
+    tag = bs.find('a', text='Browse Source')
+    if tag is not None:
+        yield 'Repository-Browse', tag.attrs['href']
+    tag = bs.find('a', text='Package Bugs')
+    if tag is not None:
+        yield 'Bug-Database', tag.attrs['href']
+    label_tag = bs.find('th', text='Homepage')
+    if label_tag is not None:
+        tag = label_tag.parent.find('a')
+        if tag is not None:
+            yield 'Homepage', tag.attrs['href']
+
+
 def guess_from_aur(package):
     vcses = ['git', 'bzr', 'hg']
     for vcs in vcses:
@@ -1417,7 +1478,7 @@ def fix_upstream_metadata(upstream_metadata):
         repo.value = url
 
 
-if __name__ == '__main__':
+def main(argv=None):
     import argparse
     import sys
     import ruamel.yaml
@@ -1438,7 +1499,9 @@ if __name__ == '__main__':
         '--consult-external-directory',
         action='store_true',
         help='Pull in external (not maintained by upstream) directory data')
-    args = parser.parse_args()
+    parser.add_argument(
+        '--version', action='version', version='%(prog)s ' + version_string)
+    args = parser.parse_args(argv)
 
     metadata = guess_upstream_metadata(
         args.path, args.trust, not args.disable_net_access,
@@ -1446,3 +1509,8 @@ if __name__ == '__main__':
         check=args.check)
 
     sys.stdout.write(ruamel.yaml.round_trip_dump(metadata))
+
+
+if __name__ == '__main__':
+    import sys
+    sys.exit(main())
